package guru.kake.yaku

import com.deepl.api.*
import javafx.beans.property.SimpleDoubleProperty
import tornadofx.getValue
import tornadofx.setValue

class TranslationBackend {
    companion object {
        var service: TranslationService? = null
        var quotaMax = 0L
        var quotaUsed = 0L
        val quotaProperty = SimpleDoubleProperty()
        var quota by quotaProperty

        operator fun invoke(text: String, sourceLang: String, destLang: String): String =
            service!!.translate(text, sourceLang, destLang)

        fun invoke(textList: List<String>, sourceLang: String, destLang: String): List<String> =
            service?.translate(textList, sourceLang, destLang) ?:
            throw TranslationServiceUnitializedException("Translation service uninitialized!")

        val provider: String = when (service) {
            null -> "uninitialized"
            is DeepLTranslator -> "DeepL"
            else -> "error"
        }
    }
}


/**
 * Interface for machine translating strings and lists of strings
 */
interface TranslationService {
    /**
     * Translate a single String of text
     */
    fun translate(text: String, sourceLang: String, destLang: String): String

    /**
     * Translate multiple Strings of text.
     *
     * Default implementation calls the single string translate() repeatedly, if the backed supports a way to translate
     * multiple strings in one request, override this function to do that instead.
     */
    fun translate(textList: List<String>, sourceLang: String, destLang: String): List<String> =
        List<String>(textList.size) {
            translate(textList[it], sourceLang, destLang)
        }
}

/**
 * TranslationsService implementation using DeepL online translation
 *
 * @param[APIkey] DeepL API key
 * @throws[DeepLException] If the API key is invalid
 */
class DeepLTranslator(APIkey: String) : TranslationService {
    private var deepl: Translator

    init {
        try {
            deepl = Translator(APIkey)
            val usage = deepl.usage
            TranslationBackend.quotaUsed = usage.character!!.count
            TranslationBackend.quotaMax = usage.character!!.limit
            TranslationBackend.quota = usage.character!!.count.toDouble() / usage.character!!.limit
        } catch (e: DeepLException) {
            throw TranslationServiceException("DeepL API initialization failed with message: ${e.message}", e)
        }
    }

    override fun translate(text: String, sourceLang: String, destLang: String): String {
        try {
            return deepl.translateText(text, sourceLang, destLang).text
        } catch (e: DeepLException) {
            throw TranslationServiceException("DeepL API translation failed with message: ${e.message}", e)
        }
    }

    override fun translate(textList: List<String>, sourceLang: String, destLang: String): List<String> {
        try {
            val translations = deepl.translateText(textList, sourceLang, destLang)
            return List<String>(translations.size) { translations[it].text }
        } catch (e: DeepLException) {
            throw TranslationServiceException("DeepL API translation failed with message: ${e.message}", e)
        }
    }
}

class TranslationServiceException(message: String? = null, cause: Throwable? = null) : Exception(message, cause)
class TranslationServiceUnitializedException(message: String? = null) : Exception(message)